package io.envoyproxy.envoymobile.engine;

import android.content.Context;

public class AndroidJniLibrary {
  // Internal reference to helper object used to load and initialize the native
  // library.
  // Volatile to ensure double-checked locking works correctly.
  private static volatile AndroidLoader loader = null;

  // Load test libraries based on the jvm_flag `envoy_jni_library_name`.
  // WARNING: This should only be used for testing.
  public static void loadTestLibrary() { JniLibrary.loadTestLibrary(); }

  public static void load(Context context) {
    if (loader != null) {
      return;
    }

    synchronized (AndroidLoader.class) {
      if (loader != null) {
        return;
      }

      JniLibrary.load();
      loader = new AndroidLoader(context);
    }
  }

  // Private helper class used by the load method to ensure the native library and
  // its dependencies are loaded and initialized at most once.
  private static class AndroidLoader {
    private AndroidLoader(Context context) {
      AndroidJniLibrary.initialize(context.getClassLoader());
    }
  }

  /**
   * Native binding to register the ConnectivityManager to C-Ares.
   *
   * @param classLoader Application's class loader.
   * @return The resulting status of the initialization.
   */
  protected static native int initialize(ClassLoader classLoader);
}
